--TEST dummy.gpr

REFS a:test.c:10:15
#  1> a:test.c:10:15 (declaration) scope=f:test.c:10:5
#  2> a:test.c:11:4 (write reference) scope=f:test.c:10:5
#  3> a:test.c:12:11 (reference) scope=f:test.c:10:5

REFS c:test.c:5:9
#  1> c:test.c:5:9 (declaration) scope=A:test.c:1:7
#  2> c:test.c:11:6 (write reference) scope=f:test.c:10:5
