# Force compilation of all ALI files from extended project
gprbuild -q -Pextended

# Override some LI files:
#   a.ali is overridden because its source is overridden
#      however, the spec is not overridden, so the project manager
#      could think the LI file in extended is still relevant
#   b.ali is overridden because it depends on a.ads
gprbuild -q -Pextending

gprbuild -q -Pdefault

./objects | tr -d '\r' | sort > out
diff -u test.out out
