--TESTSETUP gnatmake -Ptest_child_proc.gpr
--TEST -Ptest_child_proc.gpr
--  Check that we can get access to the parameter of a nested procedure
--  Requires GNAT >= 2012-08-30
DECL AA:example_main.adb:10:24
#  1> AA:example-child_proc.adb:1:31
--  Same for standard parameters
DECL Param_1:example_main.adb:8:20
#  1> Param_1:example.ads:5:22
