--TESTSETUP gnatmake -q -c dispatch_higlight.adb
OVERRIDES Process:dispatch_higlight.ads:5:13

OVERRIDDEN Process:dispatch_higlight.ads:5:13
#  1> Process:dispatch_higlight.adb:7:13

OVERRIDES Process:dispatch_higlight.adb:7:13
#  1> Process:dispatch_higlight.ads:5:13

OVERRIDDEN Process:dispatch_higlight.adb:7:13

REFS Process:dispatch_higlight.adb:14:5
#  1> Process:dispatch_higlight.adb:14:5 (dispatching call) scope=Dispatch:dispatch_higlight.ads:14:13
#  2> Process:dispatch_higlight.ads:5:13 (declaration) scope=Dispatch_Higlight:dispatch_higlight.ads:1:9

REFS_OVERRIDING Process:dispatch_higlight.adb:14:5
#  1> Process:dispatch_higlight.adb:14:5 (dispatching call) scope=Dispatch:dispatch_higlight.ads:14:13
#  2> Process:dispatch_higlight.ads:5:13 (declaration) scope=Dispatch_Higlight:dispatch_higlight.ads:1:9
#  3> Process:dispatch_higlight.adb:7:13 (body) scope=Dispatch_Higlight:dispatch_higlight.ads:1:9
#  4> Process:dispatch_higlight.adb:7:13 (declaration) scope=Dispatch_Higlight:dispatch_higlight.ads:1:9
#  5> Process:dispatch_higlight.adb:10:7 (label on end line) scope=Process:dispatch_higlight.adb:7:13
