--TESTSETUP sh compile.sh
--TESTNOCOMPILE -Pdummy.gpr
DECL html_stack_pop:htmlstack.h:9:7
#  1> html_stack_pop:htmlstack.h:9:7
BODY html_stack_pop:htmlstack.h:9:7
#  1> html_stack_pop:htmlstack.c:4:1 (body)
CALLS pop_font_face:htmlengine.c:3:13
#  1> e:htmlengine.c:3:39
#  2> font_face_stack:htmlengine.h:9:15
#  3> html_stack_is_empty:htmlstack.h:10:5
#  4> html_stack_pop:htmlstack.h:9:7
#  5> HTMLEngine:htmlengine.h:6:28
