with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Text_IO;           use Ada.Text_IO;
with GNATCOLL.Email.Parser; use GNATCOLL.Email, GNATCOLL.Email.Parser;
with GNATCOLL.VFS;          use GNATCOLL.VFS;

procedure Process_Msg is
   Msg : Message;
   S   : Unbounded_String;
begin
   Full_Parse_From_File (Create (+"msg"), Msg);
   To_String (Msg, Result => S);
   Put_Line (To_String (S));
end Process_Msg;
