import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("split")
    def test_split(self):
        self.gprbuild()
        self.runexec("obj/split", "split.expected")
