--TEST -Pdummy.gpr
-- We have both a read and a write at 3:5
REFS x:test.c:3:11
#  1> x:test.c:3:11 (declaration) scope=f:test.c:3:5
#  2> x:test.c:4:5 (reference) scope=f:test.c:3:5
#  3> x:test.c:4:5 (write reference) scope=f:test.c:3:5
#  4> x:test.c:4:13 (reference) scope=f:test.c:3:5
#  5> x:test.c:5:12 (reference) scope=f:test.c:3:5
POINTED x:test.c:3:11
#  1> S:test.h:8:15
REFS i:test.c:4:8
#  1> i:test.c:4:8 (reference) scope=f:test.c:3:5
#  2> i:test.c:4:8 (write reference) scope=f:test.c:3:5
#  3> i:test.h:4:8 (declaration) scope=S_tag:test.h:3:8
--  f has a mangled name
REFS f:test.c:3:5
#  1> f:test.c:3:5 (body)
#  2> f:test.c:3:5 (declaration)
#  3> f:test.c:3:5 (implicit reference)
