package A is
   type Root_Type is tagged null record;
   subtype Base_Type is Root_Type'Class;

   function Foo return Base_Type;

   V : Base_Type := Foo;

   subtype T is Integer;
   V2 : T;
end A;
