--TEST default.gpr

--  Not clear what this test intended to check, so we just do a few
--  random queries.

--  check that an "extern" declaration is properly linked to the body
DECL foo:ref.c:6:5
#  1> foo:ref.h:8:12

BODY foo:ref.h:8
#  1> foo:ref.c:6:5 (body)

REFS j:ref.h:2:8
#  1> j:ref.c:13:15 (reference) scope=foo:ref.h:8:12
#  2> j:ref.c:13:22 (reference) scope=foo:ref.h:8:12
#  3> j:ref.c:19:6 (write reference) scope=foo:ref.h:8:12
#  4> j:ref.c:20:6 (write reference) scope=foo:ref.h:8:12
#  5> j:ref.c:20:14 (reference) scope=foo:ref.h:8:12
#  6> j:ref.c:22:10 (reference) scope=foo:ref.h:8:12
#  7> j:ref.c:22:17 (reference) scope=foo:ref.h:8:12
#  8> j:ref.c:25:10 (reference) scope=foo:ref.h:8:12
#  9> j:ref.c:26:18 (reference) scope=foo:ref.h:8:12
# 10> j:ref.h:2:8 (declaration) scope=_rec1:ref.h:1:16
