#include "ref.h"

int x;
static void *v;

int foo (int i)
{
  struct _rec1 r1, r2;
  rec1 *r;
  r = (struct _rec1*) v;
  r = (rec1*) v;

  if (i > (r->j - r1.j))
    ;

  if (0 == r)
    return;

  r1.j  = 0 > i;
  r2.j  = r1.j;

  if (r1.j > r2.j)
    return 0 < i;

  i = r->j;
  return bar (r2.j);
  return i;
}
