--TESTSETUP gprbuild -q -Pnaming.gpr file2.foo
--TEST -Pnaming.gpr
REFS File:file.1.ada:3:5
#  1> File:file.1.ada:1:9 (declaration)
#  2> File:file.1.ada:3:5 (label on end line) scope=File:file.1.ada:1:9
#  3> File:file.2.ada:1:14 (body)
#  4> File:file.2.ada:6:5 (label on end line) scope=File:file.1.ada:1:9
#  5> File:file2.foo:1:6 (with line)
#  6> File:file2.foo:5:4 (reference) scope=Main:file2.foo:3:11

REFS foo:file.2.ada:2:13
#  1> Foo:file.1.ada:2:14 (declaration) scope=File:file.1.ada:1:9
#  2> Foo:file.2.ada:2:13 (body) scope=File:file.1.ada:1:9
#  3> Foo:file.2.ada:5:7 (label on end line) scope=Foo:file.1.ada:2:14
#  4> Foo:file2.foo:5:9 (static call) scope=Main:file2.foo:3:11
