--TESTSETUP g++ -c -fdump-xref header.hh
--TEST default.gpr

DECL operator=:header.hh
#  1> operator=:header.hh:8:8
DECL operator[]:header.hh
#  1> operator[]:header.hh:11:8
DECL operator<:header.hh
#  1> operator<:header.hh:47:5
DECL operator<<:header.hh
#  1> operator<<:header.hh:43:5
DECL operator>:header.hh
#  1> operator>:header.hh:49:5
DECL operator>>:header.hh
#  1> operator>>:header.hh:44:5
DECL operator==:header.hh
#  1> operator==:header.hh:51:5
DECL operator+:header.hh
#  1> operator+:header.hh:30:5
DECL operator+:header.hh:31
#  1> operator+:header.hh:31:5
DECL operator-:header.hh
#  1> operator-:header.hh:32:5
DECL operator-:header.hh:33
#  1> operator-:header.hh:33:5
DECL operator/:header.hh
#  1> operator/:header.hh:34:5
DECL operator*:header.hh
#  1> operator*:header.hh:18:8

