package body Example is

   type Local_Int is new Integer range 100 .. 1000;

   procedure Local_Proc (Param : in out Local_Int) is
   begin
      Param := 100;
   end Local_Proc;

   procedure Proc_1 (Param_1 : in out Int) is
   begin
      Param_1 := 1;
   end Proc_1;

end Example;
