with GNAT.OS_Lib; use GNAT.OS_Lib;

package body Extract4 is

   procedure Execute (Args : Argument_List) is
   begin
      null;
   end Execute;

   procedure Foo (Line : Integer) is
      Args : Argument_List_Access := new Argument_List'
        (new String'(Integer'Image (Line)),
         new String'("foo"));
      procedure Internal (A : Argument_List_Access) is
      begin
         null;
      end Internal;
   begin
      Execute (Args.all);
      Internal (Args);
   end Foo;
end Extract4;
