--TEST -Pdefault.gpr
REFS B:a.adb:6:17
#  1> B:a.adb:5:17 (declaration) scope=A:a.adb:4:11
#  2> B:a.adb:6:17 (reference) scope=A:a.adb:4:11
REFS B:a.adb:6:9
#fuzzy match for the entity
#  1> B:a.adb:5:17 (declaration) scope=A:a.adb:4:11
#  2> B:a.adb:6:17 (reference) scope=A:a.adb:4:11
REFS A:b.c:6:17
#  1> A:b.c:6:17 (declaration) scope=main:b.c:5:17
#  2> A:b.c:7:9 (write reference) scope=main:b.c:5:17
