--TESTSETUP gcc -c -gnat05 a.ads
COMPONENT Color:a.ads:4:9
#  1> Component:a.ads:3:9

COMPONENT Color2:a.ads:7:9
#  1> Component:a.ads:3:9

PARENT_TYPES Ta:a.ads:14:9
#  1> I1:a.ads:11:9
#  2> I2:a.ads:12:9
#  3> Parent:a.ads:13:9
