package A is
   procedure Ada_Fromada;
   pragma Export (C, Ada_Fromada, "fromada");

   procedure Ada_Fromc;
   pragma Import (C, Ada_Fromc, "fromc");

   procedure Ada_FromcppI (A : Integer);
   pragma Import (Cpp, Ada_FromcppI, "_Z7fromcppi");

   procedure Ada_FromcppF (A : Float);
   pragma Import (Cpp, Ada_FromcppF, "_Z7fromcppf");
end A;
