--TEST dummy.gpr

REFS U1:test.cc:1
#  1> U1:test.cc:1:7 (declaration)
#  2> U1:test.cc:7:9 (reference)
#  3> U1:test.cc:9:7 (reference)
#  4> U1:test.cc:21:5 (reference) scope=main:test.cc:19:5
#  5> U1:test.cc:23:11 (reference) scope=main:test.cc:19:5

CALLS U1:test.cc:1
#  1> c:test.cc:4:14
#  2> i:test.cc:2:8
#  3> j:test.cc:3:9

QNAME i:test.cc:2:8
#  1> U1::i

TYPE u_a:test.cc:9
#  1> U1:test.cc:1:7

CALLS U2:test.cc:13
#  1> T:test.cc:12:11
#  2> UU:test.cc:7:12
#  3> x:test.cc:14:11
#  4> y:test.cc:15:11
#  5> z:test.cc:16:30

