package body Dispatch_Higlight is

  package Use_Extend is new Extend;

  type My_Tagged_T is new Use_Extend.Extended_T with null record;

  procedure Process (My_Tagged : My_Tagged_T) is
  begin
    null;
  end Process;

  procedure Dispatch (E : Tagged_T'Class) is
  begin
    Process (E);
  end Dispatch;

end Dispatch_Higlight;
