with Ada.Finalization;
package Test_1 is
   type Big_Integer is tagged limited private;
   type Big_Integer2 is new Ada.Finalization.Limited_Controlled
      with null record;
private
   type Big_Integer is new Ada.Finalization.Limited_Controlled with
      record
         Value : aliased Integer;
      end record;
end;
