--TESTSETUP gnatmake -a -Pdefault
--TESTNOCOMPILE -Pdefault --runtime

--  First body is in the local file
--  Second body is in the standard runtime
--  Third body is in an alternate implementation for gnatmem
BODY Alloc:s-memory.ads:53
#  1> Alloc:s-memory.adb:32:13 (body) scope=Memory:s-memory.ads:45:16
#  2> Alloc:s-memory.adb:70:13 (body) scope=Memory:s-memory.ads:45:16
#  3> Alloc:memtrack.adb:163:13 (body) scope=Memory:s-memory.ads:45:16
