--TESTSETUP gnatmake -f -c -q nested.adb

CALLS nested:nested.adb:2:11
#  1> A:nested.adb:7:4
#  2> B:nested.adb:9:4
#  3> D:nested.adb:20:4
#  4> F:nested.adb:29:4
#  5> Nested1:nested.adb:4:14
#  6> Nested2:nested.adb:8:14

CALLS Nested1:nested.adb:11:14
#  1> C:nested.adb:15:6
#  2> G:nested.adb:5:7
#  3> H:nested.adb:6:7
#  4> Nested2:nested.adb:8:14

REFS nested1:nested.adb:11:14
#  1> Nested1:nested.adb:4:14 (declaration) scope=Nested:nested.adb:2:11
#  2> Nested1:nested.adb:11:14 (body) scope=Nested:nested.adb:2:11
#  3> Nested1:nested.adb:18:8 (label on end line) scope=Nested1:nested.adb:4:14
#  4> Nested1:nested.adb:25:7 (static call) scope=Nested2:nested.adb:8:14
#  5> Nested1:nested.adb:31:4 (static call) scope=Nested:nested.adb:2:11

REFS G:nested.adb:12:10
#fuzzy match for the entity
#  1> G:nested.adb:5:7 (declaration) scope=Nested1:nested.adb:4:14
#  2> G:nested.adb:12:7 (body) scope=Nested1:nested.adb:4:14

REFS A:nested.adb:7:4
#  1> A:nested.adb:7:4 (declaration) scope=Nested:nested.adb:2:11
