import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("bools1")
    def test_bools1(self):
        self.gprbuild()
        self.runexec("obj/test_tribools", "test_tribools.expected")
