with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package File_IO is

   function Read_File (File_Name : String) return Unbounded_String;
   procedure Write_File (File_Name : String; File_Contents : Unbounded_String);

end File_IO;
