--TEST -Pdummy.gpr
shell g++ -c -fdump-xref -fsyntax-only box.h
REFRESH
QNAME Box:box.h:21:5
#  1> Box::Box
QNAME Box:box.ads:2:14
#  1> Box.Box
QNAME Box:box.adb:7:10
#  1> Box.Box
QNAME Foo:box.adb:16:17
#  1> Box.Pack.Foo
