--TESTSETUP gnatmake -q -u lib-xref.adb
--TESTNOCOMPILE -Pdefault.gpr
CALLERS Exact_Source_Name:lib-xref.adb:2120:38
#  1> Add_ALFA_Scope:lib-xref-alfa.adb:151:14
#  2> Add_One_Xref:lib-xref-alfa.adb:621:10
#  3> Output_One_Ref:lib-xref.adb:1635:13
