--TESTSETUP gnatmake -q -p -P small.gpr
--TESTSETUP gnatmake -q -p -P Extension/extension.gpr
--TESTNOCOMPILE -PExtension/extension.gpr

-- GPS does not parse an ALI file from an extending project when the
-- corresponding source file only exists in the extended project.

REFS T_DATA:Extension/pkg.ads:6:9
#  1> T_DATA:pkg.ads:6:9 (declaration) scope=Pkg:pkg.ads:5:9
#  2> T_DATA:pkg.ads:7:26 (reference) scope=Pkg:pkg.ads:5:9
#  3> T_DATA:pkg.ads:11:9 (full declaration) scope=Pkg:pkg.ads:5:9
#  4> T_DATA:ref.adb:10:29 (reference) scope=Dummy:ref.ads:2:14
