--  vi:
--     set noexpandtab
--     set tabstop=8
procedure A is
   type         B is new Integer;  --  B at column 17, not 9
   C :		B;   --  B is at column 17, not 9
   pragma Unreferenced (C);
begin
   null;
end A;
