import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_iconv
import os


class Test(GNATCOLL_TestCase):
    @requires_iconv
    @chdir("iconv1")
    def test_iconv1(self):
        self.gprbuild()
        self.runexec("obj/test_iconv", "")
        self.runexec("obj/test_bad_charset", "")
