import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import os


class Test(GNATCOLL_TestCase):
    @chdir("pools1")
    def test_pools1(self):
        self.gprbuild()
        self.runexec("obj/testpools", "test.out")
