--TESTSETUP gprbuild -Pdefault.gpr
PARENT_TYPES Big_Integer:test_1.ads:7:9
#  1> Limited_Controlled:a-finali.ads:50:9
PARENT_TYPES Big_Integer2:test_1.ads:4:9
#  1> Limited_Controlled:a-finali.ads:50:9
