package body Primitive is

   procedure Prim (P : T) is
   begin
      null;
   end Prim;

   procedure Prim (P : T2) is
   begin
      null;
   end Prim;

   procedure Prim (P : T3) is
   begin
      null;
   end Prim;

   procedure Dispatch (P : T'Class) is
   begin
      Prim (P);
   end Dispatch;

   procedure Non_Dispatch (P : T) is
   begin
      Prim (P);
   end Non_Dispatch;

   procedure Prim (P : T4) is
   begin
      null;
   end Prim;
end Primitive;
