pragma Warnings (Off);
package body A is

   procedure Foo is separate;
   procedure My_Procedure is
      Local1 : Integer;
      Local2 : My_Record;
   begin
      My_Procedure2;
      My_Procedure3;
   end My_Procedure;

   procedure My_Procedure2 is
      Local3 : Integer;
   begin
      My_Procedure3;
   end My_Procedure2;

   procedure My_Procedure3 is
      Local1 : Integer; --  Do not mix with the one above
   begin
      My_Procedure;
   end My_Procedure3;

end A;
