--TEST default.gpr

-- Test My_Get_Num
DECL My_Get_Num:my_main.adb:8:18
#  1> My_Get_Num:my_main.adb:5:13
BODY My_Get_Num:my_main.adb:5:13
#  1> get_num:file2.c:4:5 (body)
#  2> My_Get_Num:my_main.adb:6:22 (body) scope=My_Main:my_main.adb:1:11

-- Test My_Print_Num
DECL My_Print_Num:my_main.adb:8:4
#  1> My_Print_Num:my_main.adb:2:14
BODY My_Print_Num:my_main.adb:2:14
#  1> PRINT_num:file1.c:6:6 (body)
#  2> My_Print_Num:my_main.adb:3:22 (body) scope=My_Main:my_main.adb:1:11

BODY PRINT_num:file1.h:3:6
#  1> PRINT_num:file1.c:6:6 (body)
DECL PRINT_num:file1.c:6:10
#fuzzy match for the entity
#  1> PRINT_num:file1.h:3:6
