--TESTSETUP g++ -fdump-xref -g -O2 -c main.cc CGetParam.cc Cins.cc dpu_edhipc_decl_c.cc

DOC none:dpu_ant_messages.h:185:15
#enumeration literal declared at dpu_ant_messages.h:185:15
#
#Type
#   t_weapon_class::t_weapon_class
#

TYPE none:dpu_ant_messages.h:185:15
#  1> t_weapon_class:dpu_ant_messages.h:190:24

TYPE t_weapon_class:dpu_ant_messages.h:190:24
#  1> t_weapon_class:dpu_ant_messages.h:185:14
