
// A c++ comment (1)
// on multiple lines
int cpp_before;

int cpp_after;
// A c++ comment (2)
// on multiple lines

/* A c comment (3)
   also on multiple lines */
int c_before1;

/* A c comment (4)
   also on multiple lines
*/
int c_before2;

int c_after1;
/* A c comment (5)
   also on multiple lines */

int c_after2;
/*
   A c comment (6)
   also on multiple lines
*/

/* A c comment (7) on a single line */
int c_before3;

// A c comment (8) on a single line
int c_before4;

int foo(int a) { return a; }
float foo(float a) { return a; }
