import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("read1")
    def test_read1(self):
        # Only test that we can compile
        self.gprbuild()
