procedure My_Main is
   procedure My_Print_Num (Num : Integer);
   pragma Import (C, My_Print_Num, "PRINT_num");

   function My_Get_Num return Integer;
   pragma Import (C, My_Get_Num, "get_num");
begin
   My_Print_Num (My_Get_Num);
end My_Main;
