#ifndef _HTMLSTACK_H_
#define _HTMLSTACK_H_

struct _HTMLStack {
   int foo;
};
typedef struct _HTMLStack HTMLStack;

void* html_stack_pop (HTMLStack *stack);
int html_stack_is_empty (HTMLStack *stack);

#endif /* _HTMLSTACK_H_ */
