--TESTSETUP gprbuild -Pdummy.gpr rename
--TEST -Pdummy.gpr
--  read_ali
--DUMP

--  find_ref bar
REFS bar:rename.adb:16:14
#  1> Bar:rename.adb:16:14 (declaration) scope=Rename:rename.adb:11:11
#  2> Bar:rename.adb:17:26 (reference) scope=Rename:rename.adb:11:11
#  3> Bar:rename.adb:19:27 (reference) scope=Rename:rename.adb:11:11
#  4> Bar:rename.adb:21:14 (body) scope=Rename:rename.adb:11:11
#  5> Bar:rename.adb:21:34 (label on end line) scope=Rename:rename.adb:11:11
#  6> Bar:rename.adb:48:4 (static call) scope=Rename:rename.adb:11:11

--  find_ref foo
REFS foo:rename.adb:17:14
#Renaming of Bar:rename.adb:16:14
#  1> Foo:rename.adb:17:14 (declaration) scope=Rename:rename.adb:11:11
#  2> Foo:rename.adb:44:7 (static call) scope=C:rename.adb:42:14
#  3> Foo:rename.adb:49:4 (static call) scope=Rename:rename.adb:11:11

REFS ai:rename.adb:36:4
#Renaming of Arr:rename.adb:34:4
#  1> AI:rename.adb:36:4 (declaration) scope=Rename:rename.adb:11:11
#  2> AI:rename.adb:55:4 (reference) scope=Rename:rename.adb:11:11
#  3> AI:rename.adb:68:26 (reference) scope=Rename:rename.adb:11:11

REFS y:rename.adb:40:4
#Renaming of X:rename.adb:39:4
#  1> Y:rename.adb:40:4 (declaration) scope=Rename:rename.adb:11:11
#  2> Y:rename.adb:58:4 (reference) scope=Rename:rename.adb:11:11

COMPONENT Arr:rename.adb:34:4
#  1> predefined entity: Integer
