--TESTSETUP gnatmake -q -Pnaming.gpr file2.foo
--TEST -Pnaming.gpr

IMPORTS file.2.ada
#  1> bar_.ada

REFS proc:bar_.ada:2:13
#  1> Proc:file.2.ada:6:10 (static call) scope=Foo:file.1.ada:2:14
#  2> Proc:bar.ada:2:14 (body) scope=Bar:bar_.ada:1:9
#  3> Proc:bar.ada:5:8 (label on end line) scope=Proc:bar_.ada:2:13
#  4> Proc:bar_.ada:2:13 (declaration) scope=Bar:bar_.ada:1:9
