--TEST -Pextending/extending.gpr
--  Compile in the extended project only, not the extending project
shell gnatmake -q -Pextended/extended
REFRESH

--  find all references to an entity from the extending
--  project should also find references in the extended project.
REFS Foo:extending/b.ads:2:14
#  1> Foo:a.adb:4:6 (static call) scope=A:a.adb:2:11
#  2> Foo:b.adb:2:14 (body) scope=B:b.ads:1:9
#  3> Foo:b.adb:5:8 (label on end line) scope=Foo:b.ads:2:14
#  4> Foo:b.ads:2:14 (declaration) scope=B:b.ads:1:9
