--TESTSETUP gprbuild -ws -Pdummy -q -cargs:C++ -fsyntax-only -fdump-xref
--TEST -Pdummy.gpr

-- Testing unresolved overloaded entity from one of its reference
REFS f:test.cc:22:4
#  1> f:test.cc:2:6 (body)
#  2> f:test.cc:2:6 (declaration)
#  3> f:test.cc:2:6 (implicit reference)
#  4> f:test.cc:22:4 (static call) scope=main:test.cc:20:5

-- Testing unresolved overloaded entity not from a reference
REFS f:test.cc:22:5
#fuzzy match for the entity
#  1> f:test.cc:2:6 (body)
#  2> f:test.cc:2:6 (declaration)
#  3> f:test.cc:2:6 (implicit reference)
#  4> f:test.cc:22:4 (static call) scope=main:test.cc:20:5

-- Testing unresolved overloaded entity from its body
REFS g:test.cc:6:6
#  1> g:test.cc:4:6 (declaration)
#  2> g:test.cc:4:6 (implicit reference)
#  3> g:test.cc:6:6 (body)
#  4> g:test.cc:24:4 (static call) scope=main:test.cc:20:5

-- Testing unresolved overloaded entity from its declaration
REFS g:test.cc:4:6
#  1> g:test.cc:4:6 (declaration)
#  2> g:test.cc:4:6 (implicit reference)
#  3> g:test.cc:6:6 (body)
#  4> g:test.cc:24:4 (static call) scope=main:test.cc:20:5

-- Testing the call graph from a reference to overloaded entity
CALLERS g:test.cc:24:4
#  1> main:test.cc:20:5

-- Testing the call graph from a declaration of an overloaded entity
CALLERS g:test.cc:4:6
#  1> main:test.cc:20:5

-- Testing the call graph from a body of an overloaded entity
CALLERS g:test.cc:6:6
#  1> main:test.cc:20:5
