--TESTSETUP gnatmake -q -c -Pentity
--TEST -Pentity.gpr
CHILD_TYPES_RECURSIVE root:lib.ads:3:9
#  1> Operation:lib2.ads:5:9
#  2> Action:lib3.ads:5:9
--  Only direct children
CHILD_TYPES root:lib.ads:3:9
#  1> Operation:lib2.ads:5:9
CHILD_TYPES operation:lib2.ads:5:9
#  1> Action:lib3.ads:5:9
