--TEST -Pdummy.gpr

-- This test checks that we correctly handle C++ files with special naming
-- schemes (both a.c and test.cc for bodies)

REFS f:a.c:3:5
#  1> f:a.c:3:5 (body)
#  2> f:a.c:3:5 (declaration)
#  3> f:a.c:3:5 (implicit reference)

BODY f:test.h:6:8
#  1> f:a.c:5:8 (body)
