package A is
   type Index_Type is (Red, Green, Blue);
   type Component is mod 256;
   type Color is array (Index_Type) of Component;

   type Index_Type2 is range 1 .. 3;
   type Color2 is array (Index_Type2, Integer range 1 .. 3) of Component;

   type Color3 is array (Integer range 1 .. 3) of Integer;

   type I1 is interface;
   type I2 is interface;
   type Parent is tagged null record;
   type Ta is new Parent and I1 and I2 with null record;
end A;
