package body Box is
   J : Integer := 10;
   procedure Box is
   begin
      if J >= 0 then
         J := J - 1;
         Box;
      end if;
   end Box;

   package Pack is
      procedure Foo;
   end Pack;

   package body Pack is
      procedure Foo is
      begin
         null;
      end Foo;
   end Pack;
end Box;
