--TEST default.gpr

--  The exact initial intent for this test is unclear, so we just do a
--  number of random queries.

--  The .h and the .c files do not have the same base name. Check that we
--  properly match spec and body from one to the other.

DECL foo:ref3.c:3
#  1> foo:ref.h:8:12

BODY foo:ref.h:8
#  1> foo:ref3.c:3:5 (body)

REFS j:ref.h:2
#  1> j:ref.h:2:8 (declaration) scope=_rec1:ref.h:1:16
#  2> j:ref3.c:8:6 (write reference) scope=foo:ref.h:8:12
#  3> j:ref3.c:9:6 (write reference) scope=foo:ref.h:8:12
#  4> j:ref3.c:9:14 (reference) scope=foo:ref.h:8:12
#  5> j:ref3.c:11:18 (reference) scope=foo:ref.h:8:12
