shell gcc -c binary.adb
REFRESH
BODY =:binary.ads:5:14
#  1> =:binary.adb:3:14 (body) scope=Binary:binary.ads:1:9

--  same with approximate location
BODY =:binary.ads:5:13
#fuzzy match for the entity
#  1> =:binary.adb:3:14 (body) scope=Binary:binary.ads:1:9
--  test other operators
BODY <=:binary.ads:6:14
#  1> <=:binary.adb:8:14 (body) scope=Binary:binary.ads:1:9
BODY >=:binary.ads:7:14
#  1> >=:binary.adb:13:14 (body) scope=Binary:binary.ads:1:9
BODY <:binary.ads:8:14
#  1> <:binary.adb:18:14 (body) scope=Binary:binary.ads:1:9
BODY >:binary.ads:9:14
#  1> >:binary.adb:23:14 (body) scope=Binary:binary.ads:1:9

--  Other operations on operators
PARAMS =:binary.ads:5:14
#  1> Left:binary.ads:5:18 (IN_PARAMETER)
#  2> Right:binary.ads:5:24 (IN_PARAMETER)
REFS =:binary.ads:5:14
#  1> =:binary.adb:3:14 (body) scope=Binary:binary.ads:1:9
#  2> =:binary.adb:6:9 (label on end line) scope==:binary.ads:5:14
#  3> =:binary.ads:5:14 (declaration) scope=Binary:binary.ads:1:9
TYPE <:binary.ads:8:14
#  1> predefined entity: Boolean
