
--  File: Test 1

package A1 is
   procedure Foo;
end A1;

with B1, B2;
package body A1 is
   procedure Foo is
   begin
      B1.Bar;
      B2.Bar;
   end Foo;
end A1;

package B1 is
   procedure Bar;
end B1;

package body B1 is
   procedure Bar is
   begin
      null;
   end Bar;
end B1;


