pragma Ada_05;
package Pkg is
   type T is abstract tagged null record;
   procedure Prim (Obj : T) is abstract;

   type DT is new T with null record;
   procedure Prim (Obj : DT) is null;
end Pkg;

