with Ada.Text_IO; use Ada.Text_IO;

with GNATCOLL.Mmap; use GNATCOLL.Mmap;

procedure Is_Mmap_Available is
   F : Mapped_File := Open_Read ("empty.txt");
begin
   Put_Line
     (Boolean'Image (Is_Mmapped (F)));
   Close (F);
end Is_Mmap_Available;
