package A is

  --  Some documentation for Doc_Before
  --  On multiple lines of course
  type Doc_Before is new Integer;

  type Doc_After is new Integer;
  --  Some documentation for Doc_After
  --  Still on multiple lines
  --
  --  Even multiple blocks of documentation

  --  Some documentation for Doc_Both
  --  Why not on multiple lines ?
  type Doc_Both is new Integer;
  --  Also some doc after

  type No_Doc is new Integer;


  --  Some documentation for a subprogram_before
  --  On multiple lines for a change
  procedure Subprogram_Before
     (A : Integer;
      B : Integer);

  procedure Subprogram_After
     (A : Integer;
      B : Integer);
  --  Some documentation for a subprogram_after
  --  On multiple lines for a change

  procedure Subprogram_No_Doc;

  procedure Doc_On_Body;

  --  Doc should be ignored, and is not for Doc_On_Body
end A;
