--TESTSETUP gcc -c ibusiness_unit-test.adb

--  Explicitly compile a separate, so we now have a specific ALI file
--TESTSETUP gcc -c -gnatc ibusiness_unit-test-get.adb

DECL fire:ibusiness_unit-test-get.adb:4:17
#  1> fire:ibusiness_unit-fm_manager.ads:35:13
