--TEST -Pdummy.gpr
--  find all references to an entity
REFS proc1:main.adb:4:6
#  1> Proc1:a.adb:4:14 (body) scope=A:a.ads:1:9
#  2> Proc1:a.adb:7:8 (label on end line) scope=Proc1:a.ads:2:14
#  3> Proc1:a.ads:2:14 (declaration) scope=A:a.ads:1:9
#  4> Proc1:b.adb:5:9 (static call) scope=Proc1:b.ads:2:14
#  5> Proc1:main.adb:4:6 (static call) scope=Main:main.adb:2:11

--  Params of Proc1
PARAMS proc1:main.adb:4:6
#  1> A:a.ads:2:21 (IN_PARAMETER)
#  2> C:a.ads:2:24 (IN_PARAMETER)
#  3> F:a.ads:2:37 (IN_PARAMETER)

--  Same for another entity
REFS proc1:main.adb:5:6
#  1> Proc1:a.adb:6:9 (static call) scope=Proc1:a.ads:2:14
#  2> Proc1:a.adb:11:9 (static call) scope=Proc2:a.ads:3:14
#  3> Proc1:b.adb:3:14 (body) scope=B:b.ads:1:9
#  4> Proc1:b.adb:6:8 (label on end line) scope=Proc1:b.ads:2:14
#  5> Proc1:b.ads:2:14 (declaration) scope=B:b.ads:1:9
#  6> Proc1:main.adb:5:6 (static call) scope=Main:main.adb:2:11
