--TESTSETUP gnatmake -q -gnatdM a
--  Check exact match on reference
DECL foo:a.adb:8:9
#  1> Foo:b.ads:2:14

--  Check exact match on declaration
DECL bar:a.ads:4:14
#  1> Bar:a.ads:4:14

--  Check exact match in separate
DECL b:a~bar.adb:6:4
#  1> B:b.ads:1:9

--  Check fuzzy match
DECL foo:a.adb:8:10
#fuzzy match for the entity
#  1> Foo:b.ads:2:14

--  Check bodies
BODY bar:a.ads:4:14
#  1> Bar:a.adb:4:14 (body) scope=A:a.ads:2:9
#  2> Bar:a~bar.adb:4:11 (body)

--  Check body from reference in other file
BODY foo:a~bar.adb:6:6
#  1> Foo:b.adb:4:14 (body) scope=B:b.ads:1:9

--  Check dependencies on separates (1)
IMPORTING a~bar.adb
IMPORTS a~bar.adb

--  Check dependencies when we don't have ALI file
IMPORTS d.ads
IMPORTS a-toto.adb

--  Check dependencies on separates (2)
IMPORTING b-bar.adb
IMPORTS b-bar.adb

--  Check dependencies on specs (no a-toto*, since we don't have ALI file)
IMPORTING a.ads
#  1> b.adb
#  2> c.adb
#  3> c.ads
IMPORTS a.ads
#  1> b.ads
