--TESTSETUP gprbuild -u  -Pdefault.gpr 

REFS configure:foo.hpp
#  1> configure:foo.cpp:3:11 (body)
#  2> configure:foo.cpp:12:6 (static call) scope=main:foo.cpp:10:5
#  3> configure:foo.hpp:5:9 (declaration) scope=operator==:foo.hpp:4:9
#  4> configure:foo.hpp:5:9 (implicit reference) scope=operator==:foo.hpp:4:9

REFS configure:foo.cpp
#  1> configure:foo.cpp:3:11 (body)
#  2> configure:foo.cpp:12:6 (static call) scope=main:foo.cpp:10:5
#  3> configure:foo.hpp:5:9 (declaration) scope=operator==:foo.hpp:4:9
#  4> configure:foo.hpp:5:9 (implicit reference) scope=operator==:foo.hpp:4:9
