with GNATCOLL.Scripts;         use GNATCOLL.Scripts;

package Support is

   type Cache_Data is record
      Data : Integer := 0;
      Inst : Instance_List_Access;
   end record;
   type Cache_Data_Access is access all Cache_Data;

   function Lookup return Cache_Data_Access;
   procedure Destroy (Self : in out Cache_Data_Access);

   procedure Set_Data (CI : in out Class_Instance; Data : Cache_Data_Access);
   function Get_Data (CI : Class_Instance) return Cache_Data_Access;

end Support;
