--TESTSETUP gcc -c -gnatc p.ads
--TESTNOCOMPILE default.gpr

leading_doc := false

DOC p:p.ads
#package declared at p.ads:6:9
#
#!
#! Purpose
#!   Initialisiert und verwaltet die ILGD User.
#!
#! Initialisation Exceptions
#!   keine
#!
#! Notes
#!   keine
#

DOC set_db_info:p.ads
#procedure declared at p.ads:28:14
#
#!
#! Purpose
#!   Setzt die DB-Info des Connection-Pools.
#!
#! Exceptions
#!   keine
#!
#! Notes
#!   Die DB-Info muss vor dem ersten Aufruf von Get_connection
#!   gesetzt werden.
#
#
#Parameters
#   Sql_Connection_Pool : in out P.T_SQL_CONNECTION_POOL
#   Db_Info : in P.T_DB_INFO
#

DOC is_user_defined:p.ads
#function declared at p.ads:48:13
#
#!
#! Purpose
#!   Gibt zurueck, ob ein entsprechender ILTIS USER defniniert wurde.
#!
#! Exceptions
#!   keine
#!
#! Notes
#!   keine
#
#
#Parameters
#   Kind : in P.T_ILTIS_USER_KIND
#   Master : in Boolean
#Returns
#   Boolean
#

DOC foo:p.ads
#procedure declared at p.ads:63:14
#
#comments for Foo
#Parameters
#   A : in Integer
#   B : in Integer
#

DOC foo2:p.ads
#procedure declared at p.ads:69:14
#
#comments for Foo2
#Parameters
#   A : in Integer
#   B : in Integer
#

DOC bar:p.ads 
#procedure declared at p.ads:74:14
#
#some parameter
#Parameters
#   A : in Integer
#   B : in Integer
#
