--TESTSETUP gnatmake -q -p -Pbuild.gpr
--TESTSETUP gnatmake -q -Pdefault.gpr -c main.adb
--TESTNOCOMPILE -Pdefault.gpr
METHOD_OF Prim:main.adb:6:8
#  1> T:pkg.ads:2:8
METHOD_OF Prim:main.adb:7:7
#  1> T:pkg.ads:2:8
OVERRIDES Prim:main.adb:7:7
OVERRIDDEN Prim:main.adb:7:7
#  1> Prim:pkg.ads:6:13
OVERRIDDEN_RECURSIVE Prim:main.adb:7:7
#  1> Prim:pkg.ads:6:13
shell ./obj/test_method
#  Entity: main.adb:7:7
#    Kind: dispatching call
#Overview: procedure declared at pkg.ads:3:13
#Method_Of: record type declared at pkg.ads:2:8
#--  
#
#procedure declared at pkg.ads:6:13
#Method_Of: record type declared at pkg.ads:5:8
#-- end of program
