with GNATCOLL.JSON;   use GNATCOLL;
with Text_IO; use Text_IO;

procedure Decode is
   J : JSON.JSON_Value := JSON.Create_Object;
begin
   JSON.Set_Field(J, "NoDQ", "NoDQ");
   JSON.Set_Field(J, "WithDQ", "With""");

   declare
      J_Item : String := JSON.Write(J);
      J_Check : JSON.JSON_Value;
   begin
      Put_Line(J_Item);
      J_Check := JSON.Read(J_Item, "");
      Put_Line(JSON.Get(J_Check, "WithDQ"));
   end;
end Decode;
