with GNATCOLL.Iconv; use GNATCOLL.Iconv;
procedure Test_Bad_Charset is
   St : Iconv_T;
   Success : Boolean := False;
begin
   begin
      St := Iconv_Open ("nonexistent", "nonexistent");
   exception
      when Unsupported_Conversion =>
         Success := True;
   end;
   if not Success then
      raise Program_Error with "expected exception not raised";
   end if;
end;
