package A is

   type T1 is tagged null record;
   type T2 is new T1 with null record;
   type T3 is new T2 with null record;
   type T4 is new T3 with null record;
   type T5 is new T2 with null record;
   type T6 is new T3 with null record;

end A;
