shell gnatmake -q a
REFRESH
--  from a.adb
IMPORTS a.adb
#  1> b.ads
--  from a~foo.adb
IMPORTS a~foo.adb
--  to a.ads
IMPORTING a.ads
--  to a~foo.adb
IMPORTING a~foo.adb
--  to a~foo-foo2.adb
IMPORTING a~foo-foo2.adb
--  to b.adb
IMPORTING b.adb
--  to b.ads
IMPORTING b.ads
#  1> a.adb
#  2> a.ads
