// Box class
class Box {
public:
    unsigned long _id;		// Identifier
    float _size;		// Size
    float _extend;		// Extensibility
    const char *_type;		// type


    Box& operator = (const Box&) { return *this; }

protected:
    int _links;			// #references (>= 1)






    // Copy constructor
    Box(const Box& box):
	_id(), _size(box._size), _extend(box._extend), 
	_type(box._type), _links(1)
    {}
	
};

