#ifndef _HTMLENGINE_H_
#define _HTMLENGINE_H_

#include "htmlstack.h"

typedef struct _HTMLEngine HTMLEngine;

struct _HTMLEngine {
   HTMLStack *font_face_stack;	/* Font face stack, elements are gchar * */
};

#endif /* _HTMLENGINE_H_ */
