with GNATCOLL.Traces;  use GNATCOLL.Traces;
with Ada.Text_IO;      use Ada.Text_IO;

procedure Main is
   Me : constant Trace_Handle := Create ("UNEXPECTED");
begin
   Parse_Config_File;
   begin
      raise Constraint_Error;
   exception
      when E : others =>
         Trace (Me, E);
   end;

end Main;
