with GNATCOLL.VFS;  use GNATCOLL.VFS;

procedure Test_Links is
   File : constant Virtual_File := Create ("hello.adb");
   W     : Writable_File;
begin
   W := File.Write_File (Append => False);
   Write (W, "This is the new version");
   Close (W);
end Test_Links;
