--  This test checks for missing xref generated by GNAT for anonymous
--  access types
--TESTSETUP gcc -c -gnat05 t.ads
REFS F:t.ads:5:7
#  1> F:t.ads:5:7 (declaration) scope=R:t.ads:4:9

DOC F:t.ads:5:7
#pointer declared at t.ads:5:7
#

POINTED F:t.ads:5:7
#  1> Obj:t.ads:2:9
