--TESTSETUP gcc -c b.adb
--TESTSETUP gcc -c a.adb

DECL a:b.adb:4:4
#  1> A:a.adb:1:11

--  Need to pause to make sure the timestamps will change.
--  When we reparse the LI file, make sure we preserve the info
--  that was there before. In particular, this test should work
--  no matter whether we parse b.ali or a.ali first.
shell sleep 2
shell gcc -c b.adb
shell gcc -c a.adb
REFRESH
DECL a:b.adb:4:4
#  1> A:a.adb:1:11
