--
--  Copyright (C) 2008, AdaCore
with Ada.Text_IO; use Ada.Text_IO;
procedure A is
   function Foo return Boolean;

   function Foo return Boolean is
   begin
      return True;
   end Foo;
begin
   Put_Line (Foo'Img);
   Put_Line (Foo'Img);
end A;
