with GNATCOLL.Projects; use GNATCOLL.Projects;
with GNATCOLL.VFS;  use GNATCOLL.VFS;
with GNAT.Strings; use GNAT.Strings;
procedure Main is
   Tree : Project_Tree;

   subtype SA is String_Access;

   Naming_String      : constant SA := new String'("naming");
   Gnatls_String      : constant SA := new String'("gnatls");
   Packages_To_Check_By_Gnatls    : constant String_List_Access :=
     new String_List'((Naming_String, Gnatls_String));

begin
   Tree.Load
     (GNATCOLL.VFS.Create (+"prj2.gpr"),
      Packages_To_Check => Packages_To_Check_By_Gnatls);
end Main;
