/* This test declares abstract method.
 *
 * SN puts it to .md table and does not put to .mi table.
 * However, when this method (vam) is used somewhere, SN
 * refers it like mi symbol.
 */


class A {
	virtual void vam() = 0; // virtual abstact method
	void m();
};

void A::m() {
	this->vam();
}
