[ENTITIES.ERROR] File ilss_command_line.adb not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_io.adb not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_command_line.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_io.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_io.adb not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_lib64.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_lib64_utils.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_win32.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_win32-memory.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File ilss_win32-strings.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File stdarg.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32-utils.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32-winbase.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32-windef.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32-winerror.ads not found in project <pwd>default.gpr
[ENTITIES.ERROR] File win32-winnt.ads not found in project <pwd>default.gpr
--TESTSETUP cp test_hex_dump.ali.save test_hex_dump.ali
--TESTNOCOMPILE -Pdefault.gpr
REFS ILSS_Hex_Dump:test_hex_dump.adb:43:30
#  1> ILSS_Hex_Dump:ilss_hex_dump.ads:36:11 (declaration)
#  2> ILSS_Hex_Dump:test_hex_dump.adb:23:6 (with line)
#  3> ILSS_Hex_Dump:test_hex_dump.adb:43:30 (reference) scope=Test_Hex_Dump:test_hex_dump.adb:31:11
