package Dispatch_Higlight is

  type Tagged_T is abstract tagged null record;

  procedure Process (T : Tagged_T) is abstract;

  generic
  package Extend is

    type Extended_T is abstract new Tagged_T with null record;

  end Extend;

  procedure Dispatch (E : Tagged_T'Class);

end Dispatch_Higlight;
