package Enum is

   --  Parent of Fruit and Apple is Enum
   type Fruit is (Apple);

   --  Parent of Sub is Enum,  Parent of T is Sub
   package Sub is
      type T is new Integer;
   end Sub;

   type Fruit3 is (
                   Pear);

   procedure Foo (A : Integer);
   procedure Bar
     (B : Integer);

private

   type Fruit2 is (Bananas);

end Enum;
