--TESTSETUP gcc -c -gnatc test.ads
PARAMS p:test.ads:2:14
#  1> i1:test.ads:2:17 (IN_PARAMETER)
#  2> i2:test.ads:2:21 (IN_PARAMETER)

PARAMS ent:test.ads:5:13
#  1> i1:test.ads:5:18 (IN_PARAMETER)
#  2> i2:test.ads:5:22 (IN_PARAMETER)
