--TESTSETUP cp src/a.c1 src/a.c
--TEST -Pdefault.gpr -XMODE=release
REFRESH
DECL f:src/a.c:3:3
#  1> f:a.c:2:7

--  Change the sources, but do not change the source directory
shell sleep 3
shell cp src/a.c2 src/a.c
shell gprbuild -ws -q -XMODE=release -cargs:C -fsyntax-only -fdump-xref
SCENARIO MODE release
REFRESH
DECL g:src/a.c:4:3
#  1> g:a.c:2:9

--  Change the sources and change the object directory
shell cp src/a.c3 src/a.c
shell gprbuild -ws -q -XMODE=debug -cargs:C -fsyntax-only -fdump-xref
SCENARIO MODE debug
REFRESH
DECL h:src/a.c:5:3
#  1> h:a.c:2:13
