--RM default.gpr default_naming.gpr default_source_list.txt
--TESTSETUP cp default.gpr.orig default.gpr
--TESTSETUP gnatname -Pdefault *.a *.adb
--TESTSETUP gnatmake -q -Pdefault main
--TEST -Pdefault.gpr
BODY bar3:main.adb:5:8
#  1> Bar3:foo.a:9:14 (body) scope=Foo:foo.a:1:9
#  2> Bar3:sep.a:8:11 (body)
