with Example.Child_Proc;

procedure Example_Main is
   P1 : Example.Int := 10;
   AA1 : Example.Int := 4;

begin
   Example.Proc_1 (Param_1 => P1);

   Example.Child_Proc (AA => AA1);
end Example_Main;
