shell gnatmake -q -c extract4
REFRESH
REFS Execute:extract4.adb:19:7
#  1> Execute:extract4.adb:5:14 (body) scope=Extract4:extract4.ads:1:9
#  2> Execute:extract4.adb:5:14 (declaration) scope=Extract4:extract4.ads:1:9
#  3> Execute:extract4.adb:8:8 (label on end line) scope=Execute:extract4.adb:5:14
#  4> Execute:extract4.adb:19:7 (static call) scope=Foo:extract4.ads:2:14
CALLERS Execute:extract4.adb:19:7
#  1> Extract4:extract4.ads:1:9
#  2> Foo:extract4.ads:2:14
