--TESTSETUP gnatmake -q -c main

--  scope for entity in separate, from a reference in a different unit
CALLS foo:main.adb:4:7
#  1> My_Procedure:ab.ads:8:14

--  standard scope tree
CALLS my_procedure:ab.adb:5:14
#  1> Local1:ab.adb:6:7
#  2> Local2:ab.adb:7:7
#  3> My_Procedure2:ab.ads:9:14
#  4> My_Procedure3:ab.ads:10:14
#  5> My_Record:ab.ads:3:9
