
package body A is
   procedure Foo1 (P : T) is begin null; end Foo1;
   procedure Foo2 (P : T) is begin null; end Foo2;
   procedure Foo3 (P : T'Class) is begin null; end Foo3;

   procedure Foo2 (P : T2) is begin null; end Foo2;
   procedure Foo4 (P : T2'Class) is begin null; end Foo4;
end A;
