--TESTSETUP gnatmake -q -c a.adb

--  Force computation of scope trees
REFS Proc_B:b.ads:2:14
#  1> Proc_B:a.adb:5:9 (static call) scope=Proc_A:a.ads:2:14
#  2> Proc_B:b.adb:2:14 (body) scope=B:b.ads:1:9
#  3> Proc_B:b.adb:5:8 (label on end line) scope=Proc_B:b.ads:2:14
#  4> Proc_B:b.ads:2:14 (declaration) scope=B:b.ads:1:9

REFS Proc_A:a.ads:2:14
#  1> Proc_A:a.adb:3:14 (body) scope=A:a.ads:1:9
#  2> Proc_A:a.adb:6:8 (label on end line) scope=Proc_A:a.ads:2:14
#  3> Proc_A:a.ads:2:14 (declaration) scope=A:a.ads:1:9

CALLS Proc_A:a.ads:2:14
#  1> B:b.ads:1:9
#  2> Proc_B:b.ads:2:14
