separate (Packages)
package body Separate_Package is

   --  The following package Separate_Package corresponds to file
   --  packages-separate_package-separate_package.adb
   package body Separate_Package is separate;

   procedure Set_Internal_X renames Separate_Package.Set_Internal_X;

begin
   Set_Internal_X;
end Separate_Package;
