package body A is

  procedure Subprogram_Before
     (A : Integer;
      B : Integer) is
  begin
     null;
  end Subprogram_Before;

  procedure Subprogram_After
     (A : Integer;
      B : Integer) is
  begin
     null;
  end Subprogram_After;

  procedure Subprogram_No_Doc is
  begin
     null;
  end Subprogram_No_Doc;

  --  Documentation for Doc_On_Body
  --  multiple lines
  procedure Doc_On_Body is
  begin
     null;
  end Doc_On_Body;

end A;
