--TEST -Pdefault.gpr

REFS c:a.c:9:17
#  1> c:a.c:9:17 (declaration) scope=foo:a.h:1:13
#  2> c:a.c:12:4 (reference) scope=foo:a.h:1:13

--  Test for extern keyword in C: The documentation for the subprogram
--  is provided next to its body, not next to its "extern" declaration.

DOC foo:a.h:1:13
#procedure declared at a.h:1:13
#
#Doc on foo
#blah blah blah
#Parameters
#   i : in int
#

DOC foo:a.c:14:3
#procedure declared at a.h:1:13
#
#Doc on foo
#blah blah blah
#Parameters
#   i : in int
#

