import M

print(M.hello())
print(M.hello("Manu"))

print("%1.2f" % M.print_float(1.2))

c = M.C1 (2)
print(c.method ("foo"))

print(c.id)
print(c.name)

c.prop = 5    # Calling setter
print(c.prop) # Calling getter

try:
    print(c.wo_prop) # Error
except AttributeError:
    print("Expected exception when getting wo_prop")
