--
--  File: Extension/pkg.ads
--

package Pkg is
   type T_DATA is private;
   type P_DATA is access T_DATA;

   procedure Proc;
private
   type T_DATA is record
      Data  : Integer;
      Value : Integer;
   end record;
end Pkg;
