shell gcc -c derived.ads
REFRESH
PARENT_TYPES gint:derived.ads:3:9
#  1> predefined entity: Integer
PARENT_TYPES gnat:derived.ads:4:9
#  1> Gint:derived.ads:3:9
TYPE global_gnat:derived.ads:6:4
#  1> Gnat:derived.ads:4:9
