typedef enum new_bool_enum { new_true = 0, new_false = 1 } new_bool;

enum color {
	red,
	green,
	blue
};

enum e_d {
	back,
	forward
} d;

new_bool a;
enum new_bool_enum b;
enum color c;

void f() {
	b = new_true;
	c = red;
	a = (new_bool_enum) new_bool (0);
}
