--TESTSETUP gnatmake -q -Pdefault.gpr
--  Create file that looks like a multi-unit ALI
--TESTSETUP touch foo~1.ali
--TEST -Pdefault.gpr

--  The order in which the ALI files (test.2~2.ali and test.2~4.ali)
--  for a given source file should not be relevant. So we test with
--  entities from both files.
--  from test.2~2.ali
DECL Bar:test.2.ada:13:10
#  1> Bar:test.2.ada:18:14

--  from test.2~4.ali
DECL Bar:test.2.ada:25:8
#  1> Bar:test.2.ada:18:14
