import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import os


class Test(GNATCOLL_TestCase):
    @chdir("ref1")
    def test_prj(self):
        self.gprbuild()
        self.runexec("obj/test_refcount", "test_refcount.out")

    #@chdir("ref2")
    #def test_ref2(self):
        #    self.gprbuild()
        #self.runexec("obj/test_refcount", "")
