
--  File: Test 2

package A2 is
   procedure Foo;
end A2;

with B2, B1;
package body A2 is
   procedure Foo is
   begin
      B1.Bar;
      B2.Bar;
   end Foo;
end A2;

package B2 is
   procedure Bar;
end B2;

package body B2 is
   procedure Bar is
   begin
      null;
   end Bar;
end B2;


