with GNATCOLL.Config;  use GNATCOLL.Config;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Text_IO;      use Ada.Text_IO;

procedure Test_Win is
   Ini  : INI_Parser;
   Pool : Config_Pool;
begin
   Configure (Ini);   --  optional

   Open (Ini, Argument (1));
   Fill (Pool, Ini);

   Put_Line ("name='" & Get (Pool, "db.name")
             & "' length=" & Get (Pool, "db.name")'Length'Img);
end Test_Win;
