package body B is

  procedure Subprogram_Before
     (A : Integer;
      B : Integer) is
  begin
     null;
  end Subprogram_Before;

  function Subprogram_After
     (A : Integer;
      B : Integer) return Doc_Before is
  begin
     return 1;
  end Subprogram_After;

end B;
