--TEST default.gpr

--  The .h and the .c files do not have the same base name. Check that we
--  still properly associate information from one with the other.

DECL foo:ref2.c:5:10
#  1> foo:ref.h:8:12

BODY foo:ref.h:8
#  1> foo:ref2.c:5:10 (body)

REFS j:ref.h:2:8
#  1> j:ref.h:2:8 (declaration) scope=_rec1:ref.h:1:16
#  2> j:ref2.c:9:6 (write reference) scope=foo:ref.h:8:12
#  3> j:ref2.c:10:6 (write reference) scope=foo:ref.h:8:12
#  4> j:ref2.c:10:14 (reference) scope=foo:ref.h:8:12
