--TEST dummy.gpr

QNAME Foo:namespace1.cc:1:11
#  1> Foo
QNAME foo1:namespace1.cc:2
#  1> Foo::foo1
QNAME subfoo1:namespace1.cc:4
#  1> Foo::Sub_Foo::subfoo1
QNAME Bar:namespace1.cc:8
#  1> Bar
QNAME bar:namespace1.cc:9
#  1> Bar::bar
QNAME Foo:namespace1.cc:14
#  1> Foo
QNAME foo2:namespace1.cc:13
#  1> Foo::foo2
