rm -f objects/gnatinspect.db

echo "============ Manual configuration ============="
gprconfig --batch --config=Auto_Text_IO --config=ada --db XML/
gnatinspect -Psal.gpr --config=default.cgpr -c "child_types Device_Type:gds-mil_std_1553-devices.ads"

echo "============ Reuse Autoconf ================="
gnatinspect -Psal.gpr --autoconf --configdb=XML/ --config=default.cgpr -c "child_types Device_Type:gds-mil_std_1553-devices.ads"

echo "============ Autoconf ================="
rm -f default.cgpr
rm -f auto.cgpr
gnatinspect -Psal.gpr --autoconf --configdb=XML/ -c "child_types Device_Type:gds-mil_std_1553-devices.ads"

echo "============ No autoconf ================="
rm -f auto.cgpr
gnatinspect -Psal.gpr -c "child_types Device_Type:gds-mil_std_1553-devices.ads"
