with GNAT.Strings;   use GNAT.Strings;
with GNATCOLL.JSON;  use GNATCOLL.JSON;
with GNATCOLL.VFS;   use GNATCOLL.VFS;
with Ada.Text_IO;    use Ada.Text_IO;

procedure Read is
   File : Virtual_File := Create ("test.json");
   Str : String_Access := File.Read_File;
   Val : JSON_Value;
begin
   Val := Read (Str.all, "test");

   Put_Line (Write (Val, Compact => False));

   Free (Str);
end Read;
