--TESTSETUP cp a.ads.1 a.ads
--RM obj/a.ali obj/b.ali
--TEST -Pdefault.gpr

refs Foo:a.ads
#  1> Foo:a.adb:3:14 (body) scope=A:a.ads:1:9
#  2> Foo:a.adb:6:8 (label on end line) scope=Foo:a.ads:3:14
#  3> Foo:a.ads:3:14 (declaration) scope=A:a.ads:1:9
#  4> Foo:b.adb:5:6 (static call) scope=B:b.adb:3:11

shell sleep 2
shell cp a.ads.2 a.ads
shell gprbuild -q -f -c -Pdefault.gpr a.adb
refresh

refs Foo:a.ads
#  1> Foo:a.adb:3:14 (body) scope=A:a.ads:1:9
#  2> Foo:a.adb:6:8 (label on end line) scope=Foo:a.ads:4:14
#  3> Foo:a.ads:4:14 (declaration) scope=A:a.ads:1:9

--  The above has lost the following. This is a limitation, which for now
--  we do not know how to lift !
--  4> Foo:b.adb:5:6 (static call) scope=B:b.adb:3:11
