with GNATCOLL.Traces;     use GNATCOLL.Traces;
with GNATCOLL.Projects;   use GNATCOLL.Projects;
with GNATCOLL.VFS;        use GNATCOLL.VFS;
with GNAT.Strings;
procedure Main is
   Tree    : Project_Tree;
   Env     : Project_Environment_Access;
   GNAT_Version : GNAT.Strings.String_Access;
begin
   GNATCOLL.Traces.Parse_Config_File;
   Initialize (Env);
   Env.Set_Path_From_Gnatls ("gnatls", GNAT_Version);
   Tree.Load (GNATCOLL.VFS.Create ("trdi_local.gpr"));

exception
   when Invalid_Project =>
      null;
end Main;
