--TESTSETUP cp src/a.adb1 src/a.adb
--TEST -Pdefault.gpr -XMODE=release
shell gnatmake -q -f -Pdefault -XMODE=release -u a.adb
REFRESH

DECL v:src/a.adb:4:6
#  1> V:b.ads:3:4

--  Change the sources, but do not change the source directory
shell sleep 2
shell cp src/a.adb2 src/a.adb
shell gnatmake -q -f -Pdefault -XMODE=release -u a.adb
REFRESH
DECL f:src/a.adb:7:4
#  1> F:a.adb:3:4

--  Change the sources and the object directory
SCENARIO MODE debug
shell cp src/a.adb3 src/a.adb
shell gnatmake -q -f -Pdefault -XMODE=debug -u a.adb
REFRESH
DECL g:src/a.adb:8:4
#  1> G:a.adb:3:7
