shell gnatmake -q -gnatdM a
REFRESH

--  Files that depend on a.ads
IMPORTING a.ads
#  1> a.adb
#  2> c.adb
#  3> c.ads
#  4> d.adb

--  Files that depend on a.adb
IMPORTING a.adb

--  a.ads depends on
IMPORTS a.ads
#  1> b.ads

--  a.adb depends on
IMPORTS a.adb
#  1> a.ads
#  2> b.ads
#  3> c.ads
#  4> d.ads

--  Testing recursive dependencies
IMPORTS b.adb
#  1> c.ads

DEPENDS b.adb
#  1> a.ads
#  2> b.ads
#  3> c.ads
