--TEST -Pdummy.gpr
BODY f:test.cc:9:8
#  1> f:test.cc:9:8 (body) scope=A:test.cc:3:7
REFS fff:test.cc:39:8
#  1> fff:test.cc:39:8 (declaration) scope=CCC:test.cc:28:7
#  2> fff:test.cc:50:6 (static call) scope=main:test.cc:48:5
REFS BB:test.cc:20
#  1> BB:test.cc:20:16 (declaration)
CHILD_TYPES BB:test.cc:20:16
#  1> BB:test.cc:23:3
REFS BB:test.cc:23
#  1> BB:test.cc:23:3 (declaration) scope=BB:test.cc:20:16
#  2> BB:test.cc:25:1 (reference)
PARENT_TYPES BB:test.cc:23
#  1> BB:test.cc:20:16
DECL CCC:test.cc
#  1> CCC:test.cc:28:7
METHODS CCC:test.cc
#  1> fff:test.cc:39:8
DECL My_Class:test.cc
#  1> My_Class:test.cc:14:12
PARENT_TYPES My_Class:test.cc
#  1> A:test.cc:3:7
DECL A:test.cc:3:7
#  1> A:test.cc:3:7
METHODS A:test.cc:3:7
#  1> f:test.cc:9:8
