--TESTSETUP gnatmake -q -gnatdM a
REFS t:a.ads:2:9
#  1> T:a.adb:3:24 (reference) scope=Foo1:a.ads:7:14
#  2> T:a.adb:4:24 (reference) scope=Foo2:a.ads:8:14
#  3> T:a.adb:5:24 (reference) scope=Foo3:a.ads:9:14
#  4> T:a.ads:2:9 (declaration) scope=A:a.ads:1:9
#  5> T:a.ads:7:24 (reference) scope=Foo1:a.ads:7:14
#  6> T:a.ads:8:24 (reference) scope=Foo2:a.ads:8:14
#  7> T:a.ads:9:24 (reference) scope=Foo3:a.ads:9:14
#  8> T:a.ads:11:19 (reference) scope=T2:a.ads:11:9

REFS t2:a.ads:11:9
#  1> T2:a.adb:7:24 (reference) scope=Foo2:a.ads:15:14
#  2> T2:a.adb:8:24 (reference) scope=Foo4:a.ads:16:14
#  3> T2:a.ads:11:9 (declaration) scope=A:a.ads:1:9
#  4> T2:a.ads:15:24 (reference) scope=Foo2:a.ads:15:14
#  5> T2:a.ads:16:24 (reference) scope=Foo4:a.ads:16:14

ENTITIES a.ads
#  1> A:a.ads:1:9
#  2> T:a.ads:2:9
#  3> F1:a.ads:3:7
#  4> F2:a.ads:4:7
#  5> Foo1:a.ads:7:14
#  6> P:a.ads:7:20
#  7> Foo2:a.ads:8:14
#  8> P:a.ads:8:20
#  9> Foo3:a.ads:9:14
# 10> P:a.ads:9:20
# 11> T2:a.ads:11:9
# 12> F3:a.ads:12:7
# 13> Foo2:a.ads:15:14
# 14> P:a.ads:15:20
# 15> Foo4:a.ads:16:14
# 16> P:a.ads:16:20
