shell gnatmake -q -u -c packages
REFRESH

--  Check we have references in separate packages, when starting from
--  separate package
REFS Separate_Package:packages.adb:11:24
#  1> Separate_Package:packages-separate_package-separate_package.adb:1:20 (reference)
#  2> Separate_Package:packages-separate_package.adb:2:14 (body)
#  3> Separate_Package:packages-separate_package.adb:12:5 (label on end line) scope=Separate_Package:packages.adb:5:12
#  4> Separate_Package:packages.adb:5:12 (declaration) scope=Packages:packages.ads:1:9
#  5> Separate_Package:packages.adb:11:8 (label on end line) scope=Packages:packages.ads:1:9
#  6> Separate_Package:packages.adb:13:17 (body) scope=Packages:packages.ads:1:9
#  7> Separate_Package:packages.adb:23:7 (reference) scope=Return_99:packages.ads:3:13
#  8> Separate_Package:packages.adb:24:7 (reference) scope=Return_99:packages.ads:3:13

REFS Separate_Package:packages.adb:6:15
#  1> Separate_Package:packages-separate_package-separate_package.adb:2:14 (body)
#  2> Separate_Package:packages-separate_package-separate_package.adb:11:5 (label on end line) scope=Separate_Package:packages.adb:6:15
#  3> Separate_Package:packages-separate_package.adb:6:17 (body) scope=Separate_Package:packages.adb:5:12
#  4> Separate_Package:packages-separate_package.adb:8:37 (reference) scope=Separate_Package:packages.adb:5:12
#  5> Separate_Package:packages.adb:6:15 (declaration) scope=Packages:packages.ads:1:9
#  6> Separate_Package:packages.adb:8:11 (label on end line) scope=Separate_Package:packages.adb:6:15
#  7> Separate_Package:packages.adb:24:24 (reference) scope=Return_99:packages.ads:3:13

REFS Separate_procedure:packages.adb:18:14
#  1> Separate_Procedure:packages-separate_procedure.adb:2:11 (body)
#  2> Separate_Procedure:packages-separate_procedure.adb:5:5 (label on end line) scope=Separate_Procedure:packages.adb:18:14
#  3> Separate_Procedure:packages.adb:18:14 (declaration) scope=Packages:packages.ads:1:9
