with Pkg;
package body Ref is
   package MXTCD renames Pkg;

   MXT_Data : MXTCD.P_DATA;
   pragma Unreferenced (MXT_Data);

   procedure Dummy is
   begin
      MXT_Data := new MXTCD.T_DATA; -- Missing reference
   end Dummy;
end Ref;
