
package Long_File_Name is
   type My_Record is record
      Field1 : Integer;
      Field2 : Float;
   end record;

   procedure My_Procedure;
   procedure My_Procedure2;
   procedure My_Procedure3;
end Long_File_Name;
