--TESTSETUP gcc -c -gnatc enum.ads
--TESTNOCOMPILE default.gpr

CALLS fruit:enum.ads:4:9
#  1> Apple:enum.ads:4:19
QNAME fruit:enum.ads:4:9
#  1> Enum.Fruit
LITERALS fruit:enum.ads:4:9
#  1> Apple:enum.ads:4:19
DOC fruit:enum.ads:4:9
#enumeration type declared at enum.ads:4:9
#
#Parent of Fruit and Apple is Enum
#

CALLS fruit2:enum.ads:20:9
#  1> Bananas:enum.ads:20:20
QNAME fruit2:enum.ads:20:9
#  1> Enum.Fruit2
LITERALS fruit2:enum.ads:20:9
#  1> Bananas:enum.ads:20:20
DOC fruit2:enum.ads:20:9
#enumeration type declared at enum.ads:20:9
#

CALLS sub:enum.ads:7:12
#  1> T:enum.ads:8:12
QNAME sub:enum.ads:7:12
#  1> Enum.Sub
DOC sub:enum.ads:7:12
#package declared at enum.ads:7:12
#
#Parent of Sub is Enum,  Parent of T is Sub
#

QNAME t:enum.ads:8:12
#  1> Enum.Sub.T
DOC t:enum.ads:8:12
#integer type declared at enum.ads:8:12
#
#Parent of Sub is Enum,  Parent of T is Sub
#
