
--  Some documentation before a package
package B is

  --  Some single line documentation before a type
  type Doc_Before is new Integer;

  type Doc_After is new Integer;
  --  Some single line documentation after a type

  --  Some single line documentation before a subprogram
  procedure Subprogram_Before
     (A : Integer;
      B : Integer);

  function Subprogram_After
     (A : Integer;
      B : Integer) return Doc_Before;
  --  Some single line documentation after a subprogram

end B;
