--TESTSETUP gnatmake -q -c -gnatk8 main

--  Scope tree for a whole file
CALLS my_procedure:lofilnam.adb:4:14
#  1> Local1:lofilnam.adb:5:7
#  2> Local2:lofilnam.adb:6:7
#  3> My_Procedure2:lofilnam.ads:9:14
#  4> My_Procedure3:lofilnam.ads:10:14
#  5> My_Record:lofilnam.ads:3:9

-- scope tree for entity in krunched file, from other file
CALLS my_procedure:main.adb:5:19
#  1> Local1:lofilnam.adb:5:7
#  2> Local2:lofilnam.adb:6:7
#  3> My_Procedure2:lofilnam.ads:9:14
#  4> My_Procedure3:lofilnam.ads:10:14
#  5> My_Record:lofilnam.ads:3:9
