--TESTSETUP gnatmake -q primitive.adb

--  From root, including overriding
REFS_OVERRIDING Prim:primitive.ads:4:14
#  1> Prim:primitive.adb:3:14 (body) scope=Primitive:primitive.ads:1:9
#  2> Prim:primitive.adb:6:8 (label on end line) scope=Prim:primitive.ads:4:14
#  3> Prim:primitive.adb:20:7 (dispatching call) scope=Dispatch:primitive.ads:6:14
#  4> Prim:primitive.adb:25:7 (static call) scope=Non_Dispatch:primitive.ads:5:14
#  5> Prim:primitive.ads:4:14 (declaration) scope=Primitive:primitive.ads:1:9
#  6> Prim:primitive.adb:8:14 (body) scope=Primitive:primitive.ads:1:9
#  7> Prim:primitive.adb:11:8 (label on end line) scope=Prim:primitive.ads:9:14
#  8> Prim:primitive.ads:9:14 (declaration) scope=Primitive:primitive.ads:1:9
#  9> Prim:primitive.adb:13:14 (body) scope=Primitive:primitive.ads:1:9
# 10> Prim:primitive.adb:16:8 (label on end line) scope=Prim:primitive.ads:12:14
# 11> Prim:primitive.ads:12:14 (declaration) scope=Primitive:primitive.ads:1:9

--  From root, not including overriding
REFS Prim:primitive.ads:4:14
#  1> Prim:primitive.adb:3:14 (body) scope=Primitive:primitive.ads:1:9
#  2> Prim:primitive.adb:6:8 (label on end line) scope=Prim:primitive.ads:4:14
#  3> Prim:primitive.adb:20:7 (dispatching call) scope=Dispatch:primitive.ads:6:14
#  4> Prim:primitive.adb:25:7 (static call) scope=Non_Dispatch:primitive.ads:5:14
#  5> Prim:primitive.ads:4:14 (declaration) scope=Primitive:primitive.ads:1:9

--  From child, including overriding
REFS_OVERRIDING Prim:primitive.ads:9:14
#  1> Prim:primitive.adb:8:14 (body) scope=Primitive:primitive.ads:1:9
#  2> Prim:primitive.adb:11:8 (label on end line) scope=Prim:primitive.ads:9:14
#  3> Prim:primitive.ads:9:14 (declaration) scope=Primitive:primitive.ads:1:9
#  4> Prim:primitive.adb:3:14 (body) scope=Primitive:primitive.ads:1:9
#  5> Prim:primitive.adb:6:8 (label on end line) scope=Prim:primitive.ads:4:14
#  6> Prim:primitive.adb:20:7 (dispatching call) scope=Dispatch:primitive.ads:6:14
#  7> Prim:primitive.adb:25:7 (static call) scope=Non_Dispatch:primitive.ads:5:14
#  8> Prim:primitive.ads:4:14 (declaration) scope=Primitive:primitive.ads:1:9
#  9> Prim:primitive.adb:13:14 (body) scope=Primitive:primitive.ads:1:9
# 10> Prim:primitive.adb:16:8 (label on end line) scope=Prim:primitive.ads:12:14
# 11> Prim:primitive.ads:12:14 (declaration) scope=Primitive:primitive.ads:1:9

--  From child, not including overriding
REFS Prim:primitive.ads:9:14
#  1> Prim:primitive.adb:8:14 (body) scope=Primitive:primitive.ads:1:9
#  2> Prim:primitive.adb:11:8 (label on end line) scope=Prim:primitive.ads:9:14
#  3> Prim:primitive.ads:9:14 (declaration) scope=Primitive:primitive.ads:1:9
