package Binary is

   type T is new Natural;

   function "=" (Left, Right : T) return Boolean;
   function "<=" (Left, Right : T) return Boolean;
   function ">=" (Left, Right : T) return Boolean;
   function "<" (Left, Right : T) return Boolean;
   function ">" (Left, Right : T) return Boolean;

end Binary;
