package body Binary is

   function "=" (Left, Right : T) return Boolean is
   begin
      return Natural (Left) = Natural (Right);
   end "=";

   function "<=" (Left, Right : T) return Boolean is
   begin
      return Natural (Left) <= Natural (Right);
   end "<=";

   function ">=" (Left, Right : T) return Boolean is
   begin
      return Natural (Left) >= Natural (Right);
   end ">=";

   function "<" (Left, Right : T) return Boolean is
   begin
      return Natural (Left) < Natural (Right);
   end "<";

   function ">" (Left, Right : T) return Boolean is
   begin
      return Natural (Left) > Natural (Right);
   end ">";

end Binary;
