def returnString():
    return "A string"

def returnIntList():
    return [12, 34]

def returnStrList():
    return ["str1", "str2"]


class NewStyleClass(object):
    pass

class OldStyleClass:
    pass

def returnNewInstance():
    return NewStyleClass()

def returnOldInstance():
    return OldStyleClass()

def raiseError():
    raise Exception("Message")
