shell gnatmake -q -c a
REFRESH

--  Called subprograms
CALLS my_procedure3:a.adb:18:14
#  1> Local1:a.adb:19:7
#  2> My_Procedure:a.ads:8:14

--  Test with different casing
CALLS My_Procedure3:a.adb:18:14
#  1> Local1:a.adb:19:7
#  2> My_Procedure:a.ads:8:14

--  Test for a record type
CALLS my_record:a.ads:3:9
#  1> Field1:a.ads:4:7
#  2> Field2:a.ads:5:7
