--TEST -Pdefault.gpr
shell gnatmake -q -Pdefault
REFS T:dbs38_001_comm_bus_bar.ada:3:19
#  1> T:dbs38_000_comm_bus_1.ada:2:9 (declaration) scope=Comm_Bus_1:dbs38_000_comm_bus_1.ada:1:9
#  2> T:dbs38_000_comm_bus_2.ada:4:19 (reference) scope=Comm_Bus_2:dbs38_000_comm_bus_2.ada:3:9
#  3> T:dbs38_001_comm_bus_bar.ada:3:19 (reference) scope=Bar:dbs38_001_comm_bus_1.ada:4:14
