import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("templates1")
    def test_templates1(self):
        self.gprbuild()
        self.runexec("obj/templates", "")
