--TESTSETUP gnatmake -q -f -gnatk8 main

-- ======= find who calls Self, starting from a parent, krunched package
CALLERS self:sytaprop.ads:4:17
#  1> Main:main.adb:3:11
#  2> Specific:sytaprop.ads:3:12
REFS self:sytaprop.ads:4:17
#  1> Self:main.adb:4:67 (reference) scope=Main:main.adb:3:11
#  2> Self:stpropsp.adb:3:14 (body) scope=Specific:sytaprop.ads:3:12
#  3> Self:stpropsp.adb:6:8 (label on end line) scope=Self:sytaprop.ads:4:17
#  4> Self:sytaprop.ads:4:17 (declaration) scope=Specific:sytaprop.ads:3:12

-- ======= same, starting from the main file that contains the renaming
CALLERS self:main.adb:4:67
#  1> Main:main.adb:3:11
#  2> Specific:sytaprop.ads:3:12
REFS self:main.adb:4:67
#  1> Self:main.adb:4:67 (reference) scope=Main:main.adb:3:11
#  2> Self:stpropsp.adb:3:14 (body) scope=Specific:sytaprop.ads:3:12
#  3> Self:stpropsp.adb:6:8 (label on end line) scope=Self:sytaprop.ads:4:17
#  4> Self:sytaprop.ads:4:17 (declaration) scope=Specific:sytaprop.ads:3:12

-- ======= same, starting from the main, starting from the renaming entity
CALLERS self:main.adb:4:14
#  1> Main:main.adb:3:11
REFS self:main.adb:4:14
#Renaming of Self:sytaprop.ads:4:17
#  1> Self:main.adb:4:14 (declaration) scope=Main:main.adb:3:11
#  2> Self:main.adb:6:4 (static call) scope=Main:main.adb:3:11

-- ======= same, starting from separate, krunched unit
CALLERS self:stpropsp.adb:4:17
#fuzzy match for the entity
#  1> Main:main.adb:3:11
#  2> Specific:sytaprop.ads:3:12
REFS self:stpropsp.adb:4:17
#fuzzy match for the entity
#  1> Self:main.adb:4:67 (reference) scope=Main:main.adb:3:11
#  2> Self:stpropsp.adb:3:14 (body) scope=Specific:sytaprop.ads:3:12
#  3> Self:stpropsp.adb:6:8 (label on end line) scope=Self:sytaprop.ads:4:17
#  4> Self:sytaprop.ads:4:17 (declaration) scope=Specific:sytaprop.ads:3:12
