--TESTSETUP gnatmake -q -Pdummy
--TEST -Pdummy.gpr
CHILD_TYPES T1:a.ads:3:9
#  1> T2:a.ads:4:9
CHILD_TYPES T2:a.ads:4:9
#  1> T3:a.ads:5:9
#  2> T5:a.ads:7:9
#  3> U1:b.ads:5:9
CHILD_TYPES T3:a.ads:5:9
#  1> T4:a.ads:6:9
#  2> T6:a.ads:8:9
#  3> U2:b.ads:6:9
PARENT_TYPES T4:a.ads:6:9
#  1> T3:a.ads:5:9
PARENT_TYPES T5:a.ads:7:9
#  1> T2:a.ads:4:9
PARENT_TYPES T6:a.ads:8:9
#  1> T3:a.ads:5:9
